@@ -183,6 +183,8 @@ get_vol_parms_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
   guint16 vol_bitmap;
   GFileInfo *info;
 
+  guint64 bytes_free, bytes_total;
+
   reply = g_vfs_afp_connection_send_command_finish (connection, res, &err);
   if (!reply)
   {
@@ -239,8 +241,6 @@ get_vol_parms_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 
   if (vol_bitmap & AFP_VOLUME_BITMAP_EXT_BYTES_FREE_BIT)
   {
-    guint64 bytes_free;
-
     g_vfs_afp_reply_read_uint64 (reply, &bytes_free);
     g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE,
                                       bytes_free);
@@ -248,13 +248,16 @@ get_vol_parms_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 
   if (vol_bitmap & AFP_VOLUME_BITMAP_EXT_BYTES_TOTAL_BIT)
   {
-    guint64 bytes_total;
-
     g_vfs_afp_reply_read_uint64 (reply, &bytes_total);
     g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_SIZE,
                                       bytes_total);
   }
 
+  if (vol_bitmap & AFP_VOLUME_BITMAP_EXT_BYTES_FREE_BIT &&
+      vol_bitmap & AFP_VOLUME_BITMAP_EXT_BYTES_TOTAL_BIT)
+    g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_USED,
+                                      bytes_total - bytes_free);
+
   g_object_unref (reply);
   
   g_simple_async_result_set_op_res_gpointer (simple, info, g_object_unref);
@@ -460,6 +463,14 @@ g_vfs_afp_volume_open_fork (GVfsAfpVolume      *volume,
   g_return_if_fail (G_VFS_IS_AFP_VOLUME (volume));
 
   priv = volume->priv;
+  
+  if (is_root (filename))
+  {
+    g_simple_async_report_error_in_idle (G_OBJECT (volume), callback,
+                                         user_data, G_IO_ERROR, G_IO_ERROR_IS_DIRECTORY,
+                                         _("File is directory"));
+    return;
+  }
 
   comm = g_vfs_afp_command_new (AFP_COMMAND_OPEN_FORK);
   /* data fork */
