@@ -1419,7 +1419,7 @@ do_close_write (GVfsBackend *backend,
   GVfsBackendSmb *op_backend = G_VFS_BACKEND_SMB (backend);
   SmbWriteHandle *handle = _handle;
   struct stat stat_at_close;
-  int stat_res;
+  int stat_res, errsv;
   ssize_t res;
   smbc_fstat_fn smbc_fstat;
   smbc_close_fn smbc_close;
@@ -1437,10 +1437,10 @@ do_close_write (GVfsBackend *backend,
 
   if (res == -1)
     {
-      g_vfs_job_failed_from_errno (G_VFS_JOB (job), errno);
-      
+      errsv = errno;
       if (handle->tmp_uri)
     	  smbc_unlink (op_backend->smb_context, handle->tmp_uri);
+      g_vfs_job_failed_from_errno (G_VFS_JOB (job), errsv);
       goto out;
     }
 
@@ -1452,9 +1452,8 @@ do_close_write (GVfsBackend *backend,
 						 op_backend->smb_context, handle->backup_uri);
 	  if (res ==  -1)
 	    {
-              int errsv = errno;
-
-          smbc_unlink (op_backend->smb_context, handle->tmp_uri);
+              errsv = errno;
+              smbc_unlink (op_backend->smb_context, handle->tmp_uri);
 	      g_vfs_job_failed (G_VFS_JOB (job),
 				G_IO_ERROR, G_IO_ERROR_CANT_CREATE_BACKUP,
 				_("Backup file creation failed: %s"), g_strerror (errsv));
@@ -1468,8 +1467,9 @@ do_close_write (GVfsBackend *backend,
 					     op_backend->smb_context, handle->uri);
       if (res ==  -1)
 	{
+	  errsv = errno;
 	  smbc_unlink (op_backend->smb_context, handle->tmp_uri);
-	  g_vfs_job_failed_from_errno (G_VFS_JOB (job), errno);
+	  g_vfs_job_failed_from_errno (G_VFS_JOB (job), errsv);
 	  goto out;
 	}
     }
@@ -1722,6 +1722,8 @@ do_query_fs_info (GVfsBackend *backend,
 					G_FILE_ATTRIBUTE_FILESYSTEM_SIZE) ||
       g_file_attribute_matcher_matches (attribute_matcher,
 					G_FILE_ATTRIBUTE_FILESYSTEM_FREE) ||
+      g_file_attribute_matcher_matches (attribute_matcher,
+                                        G_FILE_ATTRIBUTE_FILESYSTEM_USED) ||
       g_file_attribute_matcher_matches (attribute_matcher,
 					G_FILE_ATTRIBUTE_FILESYSTEM_READONLY))
     {
@@ -1740,8 +1742,12 @@ do_query_fs_info (GVfsBackend *backend,
                *       - for linux samba hosts, f_frsize is zero and f_bsize is a real block size
                *       - for some Windows hosts (XP), f_frsize and f_bsize should be multiplied to get real block size
                */
-              g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_SIZE, st.f_bsize * st.f_blocks * ((st.f_frsize == 0) ? 1 : st.f_frsize));
-              g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE, st.f_bsize * st.f_bfree * ((st.f_frsize == 0) ? 1 : st.f_frsize));
+              guint64 size, free_space;
+              size = st.f_bsize * st.f_blocks * ((st.f_frsize == 0) ? 1 : st.f_frsize);
+              free_space = st.f_bsize * st.f_bfree * ((st.f_frsize == 0) ? 1 : st.f_frsize);
+              g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_SIZE, size);
+              g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE, free_space);
+              g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_USED, size - free_space);
               g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY, st.f_flag & SMBC_VFS_FEATURE_RDONLY);
             }
         }
