@@ -49,9 +49,13 @@ struct _GVfsUDisks2Drive
   GVfsUDisks2VolumeMonitor  *monitor; /* owned by volume monitor */
   GList                     *volumes; /* entries in list are owned by volume monitor */
 
+  /* If TRUE, the drive was discovered at coldplug time */
+  gboolean coldplug;
+
   UDisksDrive *udisks_drive;
 
   GIcon *icon;
+  GIcon *symbolic_icon;
   gchar *name;
   gchar *sort_key;
   gchar *device_file;
@@ -59,6 +63,7 @@ struct _GVfsUDisks2Drive
   gboolean is_media_removable;
   gboolean has_media;
   gboolean can_eject;
+  gboolean can_stop;
 };
 
 static void gvfs_udisks2_drive_drive_iface_init (GDriveIface *iface);
@@ -88,8 +93,8 @@ gvfs_udisks2_drive_finalize (GObject *object)
       g_object_unref (drive->udisks_drive);
     }
 
-  if (drive->icon != NULL)
-    g_object_unref (drive->icon);
+  g_clear_object (&drive->icon);
+  g_clear_object (&drive->symbolic_icon);
   g_free (drive->name);
   g_free (drive->sort_key);
   g_free (drive->device_file);
@@ -118,11 +123,23 @@ emit_changed (GVfsUDisks2Drive *drive)
   g_signal_emit_by_name (drive->monitor, "drive-changed", drive);
 }
 
+
+static gpointer
+_g_object_ref0 (gpointer object)
+{
+  if (object != NULL)
+    return g_object_ref (G_OBJECT (object));
+  else
+    return NULL;
+}
+
 static gboolean
 update_drive (GVfsUDisks2Drive *drive)
 {
+  UDisksClient *udisks_client;
   gboolean changed;
   GIcon *old_icon;
+  GIcon *old_symbolic_icon;
   gchar *old_name;
   gchar *old_sort_key;
   gchar *old_device_file;
@@ -130,7 +147,13 @@ update_drive (GVfsUDisks2Drive *drive)
   gboolean old_is_media_removable;
   gboolean old_has_media;
   gboolean old_can_eject;
+  gboolean old_can_stop;
   UDisksBlock *block;
+#if UDISKS_CHECK_VERSION(2,0,90)
+  UDisksObjectInfo *info = NULL;
+#endif
+
+  udisks_client = gvfs_udisks2_volume_monitor_get_udisks_client (drive->monitor);
 
   /* ---------------------------------------------------------------------------------------------------- */
   /* save old values */
@@ -138,27 +161,30 @@ update_drive (GVfsUDisks2Drive *drive)
   old_is_media_removable = drive->is_media_removable;
   old_has_media = drive->has_media;
   old_can_eject = drive->can_eject;
+  old_can_stop = drive->can_stop;
 
   old_name = g_strdup (drive->name);
   old_sort_key = g_strdup (drive->sort_key);
   old_device_file = g_strdup (drive->device_file);
   old_dev = drive->dev;
   old_icon = drive->icon != NULL ? g_object_ref (drive->icon) : NULL;
+  old_symbolic_icon = drive->symbolic_icon != NULL ? g_object_ref (drive->symbolic_icon) : NULL;
 
   /* ---------------------------------------------------------------------------------------------------- */
   /* reset */
 
-  drive->is_media_removable = drive->has_media = drive->can_eject = FALSE;
+  drive->is_media_removable = drive->has_media = drive->can_eject = drive->can_stop = FALSE;
   g_free (drive->name); drive->name = NULL;
   g_free (drive->sort_key); drive->sort_key = NULL;
   g_free (drive->device_file); drive->device_file = NULL;
   drive->dev = 0;
   g_clear_object (&drive->icon);
+  g_clear_object (&drive->symbolic_icon);
 
   /* ---------------------------------------------------------------------------------------------------- */
   /* in with the new */
 
-  block = udisks_client_get_block_for_drive (gvfs_udisks2_volume_monitor_get_udisks_client (drive->monitor),
+  block = udisks_client_get_block_for_drive (udisks_client,
                                              drive->udisks_drive,
                                              FALSE);
   if (block != NULL)
@@ -182,13 +208,71 @@ update_drive (GVfsUDisks2Drive *drive)
     }
   drive->can_eject = udisks_drive_get_ejectable (drive->udisks_drive);
 
-  udisks_client_get_drive_info (gvfs_udisks2_volume_monitor_get_udisks_client (drive->monitor),
+#if UDISKS_CHECK_VERSION(2,0,90)
+  {
+    UDisksObject *object = (UDisksObject *) g_dbus_interface_get_object (G_DBUS_INTERFACE (drive->udisks_drive));
+    if (object != NULL)
+      {
+        info = udisks_client_get_object_info (udisks_client, object);
+        if (info != NULL)
+          {
+            drive->name = g_strdup (udisks_object_info_get_name (info));
+            drive->icon = _g_object_ref0 (udisks_object_info_get_icon (info));
+            drive->symbolic_icon = _g_object_ref0 (udisks_object_info_get_icon_symbolic (info));
+          }
+      }
+  }
+#else
+  udisks_client_get_drive_info (udisks_client,
                                 drive->udisks_drive,
                                 NULL,         /* drive_name */
                                 &drive->name,
                                 &drive->icon,
                                 NULL,         /* media_desc */
                                 NULL);        /* media_icon */
+#endif
+
+#if UDISKS_CHECK_VERSION(2,0,90)
+  {
+    /* If can_stop is TRUE, then
+     *
+     *  - the GUI (e.g. Files, Shell) will call GDrive.stop() whenever the
+     *    user presses the Eject icon, which will result in:
+     *
+     *  - us calling UDisksDrive.PowerOff() on GDrive.stop(), which
+     *    will result in:
+     *
+     *  - UDisks asking the kernel to power off the USB port the drive
+     *    is connected to, which will result in
+     *
+     *  - Most drives powering off (especially true for bus-powered
+     *    drives such as 2.5" HDDs and USB sticks), which will result in
+     *
+     *  - Users feeling warm and cozy when they see the LED on the
+     *    device turn off (win)
+     *
+     * Obviously this is unwanted if
+     *
+     *  - the drive is using removable media (e.g. optical discs,
+     *    flash media etc); or
+     *
+     *  - the device is internal
+     *
+     * So for the latter, only do this for drives we appear *during*
+     * the login session.  Note that this heuristic has the nice
+     * side-effect that USB-attached hard disks that are plugged in
+     * when the computer starts up will not be powered off when the
+     * user clicks the "eject" icon.
+     */
+    if (!drive->is_media_removable && !drive->coldplug)
+      {
+        if (udisks_drive_get_can_power_off (drive->udisks_drive))
+          {
+            drive->can_stop = TRUE;
+          }
+      }
+  }
+#endif
 
   /* ---------------------------------------------------------------------------------------------------- */
   /* fallbacks */
@@ -203,28 +287,36 @@ update_drive (GVfsUDisks2Drive *drive)
     }
   if (drive->icon == NULL)
     drive->icon = g_themed_icon_new ("drive-removable-media");
+  if (drive->symbolic_icon == NULL)
+    drive->symbolic_icon = g_themed_icon_new ("drive-removable-media-symbolic");
 
   /* ---------------------------------------------------------------------------------------------------- */
   /* compute whether something changed */
   changed = !((old_is_media_removable == drive->is_media_removable) &&
               (old_has_media == drive->has_media) &&
               (old_can_eject == drive->can_eject) &&
+              (old_can_stop == drive->can_stop) &&
               (g_strcmp0 (old_name, drive->name) == 0) &&
               (g_strcmp0 (old_sort_key, drive->sort_key) == 0) &&
               (g_strcmp0 (old_device_file, drive->device_file) == 0) &&
               (old_dev == drive->dev) &&
-              g_icon_equal (old_icon, drive->icon)
+              g_icon_equal (old_icon, drive->icon) &&
+              g_icon_equal (old_symbolic_icon, drive->symbolic_icon)
               );
 
   /* free old values */
   g_free (old_name);
   g_free (old_sort_key);
   g_free (old_device_file);
-  if (old_icon != NULL)
-    g_object_unref (old_icon);
+  g_clear_object (&old_icon);
+  g_clear_object (&old_symbolic_icon);
 
   /*g_debug ("in update_drive(); has_media=%d changed=%d", drive->has_media, changed);*/
 
+#if UDISKS_CHECK_VERSION(2,0,90)
+  g_clear_object (&info);
+#endif
+
   return changed;
 }
 
@@ -240,12 +332,14 @@ on_udisks_drive_notify (GObject     *object,
 
 GVfsUDisks2Drive *
 gvfs_udisks2_drive_new (GVfsUDisks2VolumeMonitor  *monitor,
-                        UDisksDrive               *udisks_drive)
+                        UDisksDrive               *udisks_drive,
+                        gboolean                   coldplug)
 {
   GVfsUDisks2Drive *drive;
 
   drive = g_object_new (GVFS_TYPE_UDISKS2_DRIVE, NULL);
   drive->monitor = monitor;
+  drive->coldplug = coldplug;
 
   drive->udisks_drive = g_object_ref (udisks_drive);
   g_signal_connect (drive->udisks_drive,
@@ -304,6 +398,13 @@ gvfs_udisks2_drive_get_icon (GDrive *_drive)
   return drive->icon != NULL ? g_object_ref (drive->icon) : NULL;
 }
 
+static GIcon *
+gvfs_udisks2_drive_get_symbolic_icon (GDrive *_drive)
+{
+  GVfsUDisks2Drive *drive = GVFS_UDISKS2_DRIVE (_drive);
+  return drive->symbolic_icon != NULL ? g_object_ref (drive->symbolic_icon) : NULL;
+}
+
 static char *
 gvfs_udisks2_drive_get_name (GDrive *_drive)
 {
@@ -378,13 +479,14 @@ gvfs_udisks2_drive_can_start_degraded (GDrive *_drive)
 static gboolean
 gvfs_udisks2_drive_can_stop (GDrive *_drive)
 {
-  return FALSE;
+  GVfsUDisks2Drive *drive = GVFS_UDISKS2_DRIVE (_drive);
+  return drive->can_stop;
 }
 
 static GDriveStartStopType
 gvfs_udisks2_drive_get_start_stop_type (GDrive *_drive)
 {
-  return G_DRIVE_START_STOP_TYPE_UNKNOWN;
+  return G_DRIVE_START_STOP_TYPE_SHUTDOWN;
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -657,7 +759,7 @@ gvfs_udisks2_drive_eject_with_operation (GDrive              *_drive,
 
   /* This information is needed in GVfsDdisks2Volume when apps have
    * open files on the device ... we need to know if the button should
-   * be "Unmount Anyway" or "Eject Anyway"
+   * be "Unmount Anyway", "Eject Anyway" or "Power Off Anyway"
    */
   if (mount_operation != NULL)
     {
@@ -704,6 +806,132 @@ gvfs_udisks2_drive_eject_finish (GDrive        *drive,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+#if UDISKS_CHECK_VERSION(2,0,90)
+
+typedef struct
+{
+  GSimpleAsyncResult *simple;
+
+  GVfsUDisks2Drive *drive;
+  GMountOperation *mount_operation;
+} StopData;
+
+static void
+stop_data_free (StopData *data)
+{
+  g_object_unref (data->simple);
+  g_clear_object (&data->drive);
+  g_clear_object (&data->mount_operation);
+
+  g_free (data);
+}
+
+static void
+power_off_cb (GObject      *source_object,
+              GAsyncResult *res,
+              gpointer      user_data)
+{
+  StopData *data = user_data;
+  GError *error;
+
+  error = NULL;
+  if (!udisks_drive_call_power_off_finish (UDISKS_DRIVE (source_object), res, &error))
+    {
+      gvfs_udisks2_utils_udisks_error_to_gio_error (error);
+      g_simple_async_result_take_error (data->simple, error);
+    }
+
+  if (data->mount_operation != NULL)
+    {
+      /* If we fail send an ::aborted signal to make any notification go away */
+      if (error != NULL)
+        g_signal_emit_by_name (data->mount_operation, "aborted");
+
+      gvfs_udisks2_unmount_notify_stop (data->mount_operation);
+    }
+
+  g_simple_async_result_complete (data->simple);
+  stop_data_free (data);
+}
+
+static void
+gvfs_udisks2_drive_stop_on_all_unmounted (GDrive              *_drive,
+                                          GMountOperation     *mount_operation,
+                                          GCancellable        *cancellable,
+                                          GAsyncReadyCallback  callback,
+                                          gpointer             user_data,
+                                          gpointer             on_all_unmounted_data)
+{
+  GVfsUDisks2Drive *drive = GVFS_UDISKS2_DRIVE (_drive);
+  GVariantBuilder builder;
+  StopData *data;
+
+  data = g_new0 (StopData, 1);
+  data->simple = g_simple_async_result_new (G_OBJECT (drive),
+                                            callback,
+                                            user_data,
+                                            gvfs_udisks2_drive_stop_on_all_unmounted);
+  data->drive = g_object_ref (drive);
+  if (mount_operation != NULL)
+    data->mount_operation = g_object_ref (mount_operation);
+
+  g_variant_builder_init (&builder, G_VARIANT_TYPE_VARDICT);
+  if (mount_operation == NULL)
+    {
+      g_variant_builder_add (&builder,
+                             "{sv}",
+                             "auth.no_user_interaction", g_variant_new_boolean (TRUE));
+    }
+  udisks_drive_call_power_off (drive->udisks_drive,
+                               g_variant_builder_end (&builder),
+                               cancellable,
+                               power_off_cb,
+                               data);
+}
+
+static void
+gvfs_udisks2_drive_stop (GDrive              *_drive,
+                         GMountUnmountFlags   flags,
+                         GMountOperation     *mount_operation,
+                         GCancellable        *cancellable,
+                         GAsyncReadyCallback  callback,
+                         gpointer             user_data)
+{
+  GVfsUDisks2Drive *drive = GVFS_UDISKS2_DRIVE (_drive);
+
+  /* This information is needed in GVfsDdisks2Volume when apps have
+   * open files on the device ... we need to know if the button should
+   * be "Unmount Anyway", "Eject Anyway" or "Power Off Anyway"
+   */
+  if (mount_operation != NULL)
+    {
+      g_object_set_data (G_OBJECT (mount_operation), "x-udisks2-is-stop", GINT_TO_POINTER (1));
+      gvfs_udisks2_unmount_notify_start (mount_operation, NULL, _drive, FALSE);
+    }
+
+  /* first we need to go through all the volumes and unmount their assoicated mounts (if any) */
+  unmount_mounts (drive,
+                  flags,
+                  mount_operation,
+                  cancellable,
+                  callback,
+                  user_data,
+                  gvfs_udisks2_drive_stop_on_all_unmounted,
+                  NULL);
+}
+
+static gboolean
+gvfs_udisks2_drive_stop_finish (GDrive        *drive,
+                                GAsyncResult  *result,
+                                GError       **error)
+{
+  return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error);
+}
+
+#endif /* UDISKS_CHECK_VERSION(2,0,90) */
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 static const gchar *
 gvfs_udisks2_drive_get_sort_key (GDrive *_drive)
 {
@@ -718,6 +946,7 @@ gvfs_udisks2_drive_drive_iface_init (GDriveIface *iface)
 {
   iface->get_name = gvfs_udisks2_drive_get_name;
   iface->get_icon = gvfs_udisks2_drive_get_icon;
+  iface->get_symbolic_icon = gvfs_udisks2_drive_get_symbolic_icon;
   iface->has_volumes = gvfs_udisks2_drive_has_volumes;
   iface->get_volumes = gvfs_udisks2_drive_get_volumes;
   iface->is_media_removable = gvfs_udisks2_drive_is_media_removable;
@@ -741,6 +970,9 @@ gvfs_udisks2_drive_drive_iface_init (GDriveIface *iface)
   iface->poll_for_media_finish = gvfs_udisks2_drive_poll_for_media_finish;
   iface->start = gvfs_udisks2_drive_start;
   iface->start_finish = gvfs_udisks2_drive_start_finish;
+#endif
+
+#if UDISKS_CHECK_VERSION(2,0,90)
   iface->stop = gvfs_udisks2_drive_stop;
   iface->stop_finish = gvfs_udisks2_drive_stop_finish;
 #endif
